<?php
/*
Plugin Name: The Oracle
Plugin URI: http://soeren-weber.net/post/2005/01/02/13/
Version: 1.0
Description: Inspired by the Hello Dolly plugin, the Oracle spreads its (and your) wisdom throughout the blog.  With the Oracle you are able to randomize every parameter stored in the Wordpress options table from a list of possible options. E.g. after installing this plugin the 'blogdescription' will be taken as an example. <strong><a href="../wp-content/plugins/the-oracle.php?func=configure" title="Click here to configure this plugin">Configure the plugin</a><strong>.
Author: S&ouml;ren Weber
Author URI: http://soeren-weber.net/
*/

require_once(dirname(__FILE__).'/../../wp-config.php');

if (!class_exists('oracle'))
{
  class oracle
  {
    function oracle()
    {
      $this->install();
      if (basename(__FILE__) == basename($_SERVER['PHP_SELF']))
      {
        if ($_GET['func'] && $_GET['func'] == 'configure')
          $this->configure();
        else if ($_GET['func'] && $_GET['func'] == 'new')
          $this->new_option();
        else if ($_GET['func'] && $_GET['func'] == 'edit')
          $this->edit_option();
        else if ($_GET['func'] && $_GET['func'] == 'delete')
          $this->delete_option();
      }
      $this->randomize();
    }

    function write_page($content)
    {
      echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
        <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en">
          <head>
            <title>The Oracle Configuration</title>
            <meta http-equiv="content-type" content="text/html; charset='; bloginfo('charset'); echo '" />
          </head>
          <body>'.$content.'</body>
        </html>';
    }

    function find_out_mode_and_option(&$mode, &$option_name)
    {
      $mode = "";
      $option_name = "";
      while ($mode = current($_REQUEST)) {
        $option_name = key($_REQUEST);
        if ($mode == 'New' || $mode == 'Edit' || $mode == 'Delete' || $mode == 'Save' || $mode == 'Back')
          break;
        next($_REQUEST);
      }
    }

    function copy_option_to_oracle($option_name)
    {
      global $wpdb, $table_prefix;
      $options_table = $table_prefix . 'options';
      $oracle_table = $table_prefix . 'oracle';

      $query = "SELECT option_value FROM `$options_table` WHERE option_name='$option_name'";
      $result = $wpdb->get_row($query);
      if (!$result)
        die('Error accessing database table $options_table');
      $option_value = $result->option_value;
      $wpdb->query("INSERT INTO $oracle_table VALUES ('$option_name', '$option_value');");
    }

    function install()
    {
      global $wpdb, $table_prefix;
      $oracle_table = $table_prefix . 'oracle';

      // if plugin isn't installed, create new table and fill it with defaults
      if (!$wpdb->query("SHOW TABLES LIKE '$oracle_table'"))
      {
        $wpdb->query("CREATE TABLE `$oracle_table` (
           `option_name` varchar(64) NOT NULL,
           `option_oracle` TEXT NOT NULL DEFAULT '',
           PRIMARY KEY  (`option_name`)) TYPE=MyISAM ;");
        $this->copy_option_to_oracle('blogdescription');
      }
    }

    function configure()
    {
      global $wpdb, $table_prefix, $user_level;
      $options_table = $table_prefix . 'options';
      $oracle_table = $table_prefix . 'oracle';

      get_currentuserinfo();
      if ($user_level < 5)
        die ("Sorry, you must be at least a level 5 user to configure the Oracle");

      // select all already configured options
      $result = $wpdb->get_results("SELECT option_name FROM `$oracle_table` order by option_name");
      $configured = '';
      if (!$result)
        $configured = 'No options are configured<br />';
      else {
        $configured = '<table>';
        for ($n = 0; $n < count($result); ++$n) {
          $option_name = $result[$n]->option_name;
          $configured .= '<tr><td>'.$option_name.'</td>
            <td><form method="post" action="'.get_settings('siteurl').'/wp-content/plugins/the-oracle.php?func=edit">
            <input type="submit" name="'.$option_name.'" value="Edit" /></form></td>
            <td><form method="post" action="'.get_settings('siteurl').'/wp-content/plugins/the-oracle.php?func=delete">
            <input type="submit" name="'.$option_name.'" value="Delete" /></form></td></tr>';
        }
        $configured .= '</table>';
      }

      // select all options not already configured yet
      // we want to to a subquery but mysql is to stupid in version < 4.1 (see http://dev.mysql.com/doc/mysql/en/Rewriting_subqueries.html)
      $result = $wpdb->get_results("SELECT opt.option_name FROM `$options_table` opt
        LEFT JOIN `$oracle_table` ora ON opt.option_name = ora.option_name
        WHERE ora.option_name IS NULL
        ORDER BY opt.option_name");
      $available = '';
      if (!$result)
        $available = 'No options left to configure<br />';
      else {
        $available = '<table>';
        for ($n = 0; $n < count($result); ++$n) {
          $option_name = $result[$n]->option_name;
          $available .= '<tr><td>'.$option_name.'</td>
            <td><form method="post" action="'.get_settings('siteurl').'/wp-content/plugins/the-oracle.php?func=new">
            <input type="submit" name="'.$option_name.'" value="New" /></form></td></tr>';
        }
        $available .= '</table>';
      }

      // print out options
      $content = '<div class="wrap"><p><strong><a href="'.get_settings('siteurl').'/wp-admin/plugins.php">Back to plugin administration</a><strong></p></div>
        <div class="wrap"><p>The following options are configured:</p>'.$configured.'</div>
        <div class="wrap"><p><strong><a href="'.get_settings('siteurl').'/wp-admin/plugins.php">Back to plugin administration</a><strong></p></div>
        <div class="wrap"><p>The following options can be configured:</p>'.$available.'</div>
        <div class="wrap"><p><strong><a href="'.get_settings('siteurl').'/wp-admin/plugins.php">Back to plugin administration</a><strong></p></div>';
      $this->write_page($content);
    }

    function new_option()
    {
      global $wpdb, $table_prefix, $user_level;
      $oracle_table = $table_prefix . 'oracle';

      get_currentuserinfo();
      if ($user_level < 5)
        die ("Sorry, you must be at least a level 5 user to configure the Oracle");

      $this->find_out_mode_and_option($mode, $option_name);
      if ($mode == "New")
      {
        $this->copy_option_to_oracle($option_name);
        $this->edit_option();
      }
      else {
        $this->configure();
        return;
      }
    }

    function edit_option()
    {
      global $wpdb, $table_prefix, $user_level;
      $oracle_table = $table_prefix . 'oracle';

      get_currentuserinfo();
      if ($user_level < 5)
        die ("Sorry, you must be at least a level 5 user to configure the Oracle");

      // find out option to edit
      $this->find_out_mode_and_option($mode, $option_name);
      if ($mode == "New")
        $info = "New option <strong>$option_name</strong>";
      else if ($mode == "Edit")
        $info = "Edit option <strong>$option_name</strong>";
      else if ($mode == "Save") {
        $option_oracle = $_POST['option_oracle'];
        $query = "UPDATE $oracle_table SET option_oracle='$option_oracle' WHERE option_name='$option_name'";
        $wpdb->query($query);
        $info = "Option <strong>$option_name</strong> saved";
      }
      else {
        $this->configure();
        return;
      }

      // select option_oracle from DB
      $query = "SELECT option_oracle FROM `$oracle_table` WHERE option_name='$option_name'";
      $result = $wpdb->get_row($query);
      if (!$result)
        die('Error accessing database table $oracle_table');
      $option_oracle = $result->option_oracle;

      // display update page
      $content = '<div class="wrap">
        <p>'.$info.'</p>
        <ul><li>Write every oracle option into a separate line.</li>
        <li>The plugin will chose randomly one of this options.</li>
        <li>This will overwrite the values for this option you may have set on other admin pages.</li>
        <li>After disabling the Oracle plugin or deleting this Oracle option the last random value will stay active until you change it on the appropriate admin page.</li></ul>
        <form method="post" action="'.get_settings('siteurl').'/wp-content/plugins/the-oracle.php?func=edit">
          <textarea cols="80" rows="21" name="option_oracle" tabindex="1">'.$option_oracle.'</textarea>
          <table><tr><td><input type="submit" name="'.$option_name.'" value="Save" /></td><td><input type="submit" name="'.$option_name.'" value="Back" /></td></tr></table>
        </form></div>';
      $this->write_page($content);
    }

    function delete_option()
    {
      global $wpdb, $table_prefix, $user_level;
      $oracle_table = $table_prefix . 'oracle';

      get_currentuserinfo();
      if ($user_level < 5)
        die ("Sorry, you must be at least a level 5 user to configure the Oracle");

      $this->find_out_mode_and_option($mode, $option_name);
      if ($mode == "Delete")
      {
        $query = "DELETE FROM `$oracle_table` WHERE option_name='$option_name'";
        $wpdb->query($query);
      }
      $this->configure();
    }

    function randomize()
    {
      global $wpdb, $table_prefix;
      $options_table = $table_prefix . 'options';
      $oracle_table = $table_prefix . 'oracle';

      $result = $wpdb->get_results("SELECT option_name, option_oracle FROM `$oracle_table` order by option_name");
      if (!$result)
        // die silently
        return;
      for ($n = 0; $n < count($result); ++$n)
      {
        // split choices into lines, chose randomly and store it in the proper location in the DB
        $option_name = $result[$n]->option_name;
        $option_oracle = $result[$n]->option_oracle;
        $option_oracle = explode("\n", $option_oracle);
        $chosen = wptexturize($option_oracle[mt_rand(0, count($option_oracle)-1)]);
        $query = "UPDATE $options_table SET option_value='$chosen' WHERE option_name='$option_name'";
        $wpdb->query($query);
      }
    }
  }

  $oracle = new oracle();
}
?>